@@ -8,6 +8,7 @@
 #' @param seed              Sets random seed in R global environment
 #' @param normalize         (FALSE/TRUE) For regression, should rmse, mbe and mae be normalized using (max(y) - min(y))
 #' @param bootstrap         (FALSE/TRUE) Should a bootstrap sampling be applied. If FALSE, an n-th percent withold will be conducted
+#' @param trace             Print iterations
 #' @param ...               Additional arguments passed to Random Forests 
 #'
 #' @return  For classification a "rf.cv"", "classification" class object with the following components:
@@ -89,11 +90,14 @@
 #' @exportClass rf.cv
 #' @export 	
 rf.crossValidation <- function(x, xdata, p=0.10, n=99, seed=NULL, normalize = FALSE, 
-                               bootstrap = FALSE, ...) {
+                               bootstrap = FALSE, trace = FALSE, ...) {
   if(!any(class(x) %in% c("randomForest","list"))) stop("x is not a randomForest object")
     if(!is.null(seed)) { set.seed(seed) }
 	  if(bootstrap) cat("Bootstrap sampling is being applied,", paste("p",p,sep="="), "argument is ignored", "\n")
-  if (x$type == "unsupervised") { stop("Unsupervised classification not supported")   
+  if (x$type == "unsupervised") { stop("Unsupervised classification not supported")
+
+  ###########################################
+  #### Start regression cross-validation ####	   
   } else if (x$type == "regression") {
     cat("running:", x$type, "cross-validation", "with", n, "iterations", "\n")
 	# Validation statistics (RMSE, MBE, MAE)
@@ -128,25 +132,26 @@ rf.crossValidation <- function(x, xdata, p=0.10, n=99, seed=NULL, normalize = FA
 	y.mbe <- vector()
 	model.varExp <- vector()
 	model.mse <- vector()
-    if(bootstrap) boot.sample.size <- vector()			
-	# Start cross-validation
-    sample.size = round( (length(x$y) * p), digits=0)
+    if(bootstrap) boot.sample.size <- vector()	
+      sample.size = round( (length(x$y) * p), digits=0) #population sample size 
 	  for(i in 1:n) {
-        dat <- data.frame(y=x$y, xdata) 
-	if(!bootstrap) {	
-		sidx <- sample(1:nrow(dat), sample.size)
-	    dat.sub <- dat[-sidx,]
-        dat.cv <- dat[sidx,]		
-	} else {	
-	    dat.sub <- dat[sample(1:nrow(dat), replace=TRUE),]
-        dat.cv <- dat[which(!rownames(dat) %in% rownames(dat.sub)),]		
-    }	
+	    if(trace) cat("running iteration:", i, "\n")
+        dat <- data.frame(y=x$y, xdata)
+        # Draw random sample		
+	    if(!bootstrap) {
+	      sidx <- sample(1:nrow(dat), sample.sizes[s])   
+	        dat.sub <- dat[-sidx,]                   
+            dat.cv <- dat[sidx,]		            
+	    } else {	
+	      dat.sub <- dat[sample(1:nrow(dat), replace=TRUE),]              
+          dat.cv <- dat[which(!rownames(dat) %in% rownames(dat.sub)),]	   
+        }	
 	     rf.fit <- randomForest::randomForest(y=dat.sub[,"y"], x=dat.sub[,2:ncol(dat.sub)], ...)    
- 		 model.mse <- append(model.mse, rf.fit$mse[length(rf.fit$mse)]) 
-	     model.varExp <- append(model.varExp, round(100*rf.fit$rsq[length(rf.fit$rsq)], digits=2) )         
-		 y.rmse <- append(y.rmse, rmse(dat.cv[,"y"], stats::predict(rf.fit, newdata = dat.cv[,2:ncol(dat.cv)]), norm=normalize) )
-         y.mbe <- append(y.mbe, mbe(dat.cv[,"y"], stats::predict(rf.fit, newdata = dat.cv[,2:ncol(dat.cv)]), norm=normalize) )
-		 y.mae <- append(y.mae, mae(dat.cv[,"y"], stats::predict(rf.fit, newdata = dat.cv[,2:ncol(dat.cv)]), norm=normalize) )
+ 		   model.mse <- append(model.mse, rf.fit$mse[length(rf.fit$mse)]) 
+	         model.varExp <- append(model.varExp, round(100*rf.fit$rsq[length(rf.fit$rsq)], digits=2) )         
+		       y.rmse <- append(y.rmse, rmse(dat.cv[,"y"], stats::predict(rf.fit, newdata = dat.cv[,2:ncol(dat.cv)]), norm=normalize) )
+               y.mbe <- append(y.mbe, mbe(dat.cv[,"y"], stats::predict(rf.fit, newdata = dat.cv[,2:ncol(dat.cv)]), norm=normalize) )
+		   y.mae <- append(y.mae, mae(dat.cv[,"y"], stats::predict(rf.fit, newdata = dat.cv[,2:ncol(dat.cv)]), norm=normalize) )
          if(bootstrap) boot.sample.size <- append(boot.sample.size, nrow(dat.cv) )
 	  }		 
 	 r.cv <- list(fit.var.exp=round(100*x$rsq[length(x$rsq)], digits=2), 
@@ -158,8 +163,11 @@ rf.crossValidation <- function(x, xdata, p=0.10, n=99, seed=NULL, normalize = FA
 				  model.varExp = model.varExp )
        class(r.cv) <- c("rf.cv", "regression", "list")
      return( r.cv )
+	 
+   ###############################################
+   #### Start classification cross-validation ####
    } else if (x$type == "classification") {
-    cat("running:", x$type, "cross-validation", "with", n, "iterations", "\n")
+      cat("running:", x$type, "cross-validation", "with", n, "iterations", "\n")
     if(bootstrap) boot.sample.size <- vector()	
     classes <- as.vector(levels( x$y ))	
       sample.size = round( (length(x$y) * p) / length(x$classes), digits=0) 
@@ -168,34 +176,44 @@ rf.crossValidation <- function(x, xdata, p=0.10, n=99, seed=NULL, normalize = FA
 	      mdl.ua <- as.data.frame(array(0, dim=c(0,length(classes)))) 
 	    mdl.pa <- as.data.frame(array(0, dim=c(0,length(classes))))
       mdl.oob <- as.data.frame(array(0, dim=c(0,2+length(classes))))
-	cv.oob <- as.data.frame(array(0, dim=c(0,2+length(classes))))		
-    for(i in 1:n) {
-	if(!bootstrap) {	
-	  samp.index <- vector()
-        for(r in unique(x$y)) {			  
-          cvalue <- which(x$y == r) 	  
-          samp.index <- append(samp.index, sample(cvalue, sample.size)) 
-        }		  
-	    tx <- xdata[samp.index,]
-	    ty <- x$y[samp.index]
-	    mx <- xdata[-samp.index,]
-	    my <- x$y[-samp.index]	   
-	} else {
-        dat <- data.frame(y=x$y, xdata) 	
-	    tx <- dat[sample(1:nrow(dat), replace=TRUE),]
-		ty <- tx$y 
-		tx <- tx[,2:ncol(tx)]
-	    mx <- dat[-which(!rownames(dat) %in% rownames(tx)),]
-	    my <- mx$y
-		mx <- mx[,2:ncol(mx)] 
-    }		  
-    rf.fit <- randomForest::randomForest(y=as.factor(my), x=mx, ytest=as.factor(ty), xtest=tx, ...)        
+	cv.oob <- as.data.frame(array(0, dim=c(0,2+length(classes))))	
+	
+    # Create class-level sample sizes
+	nclass <- length(unique(x$y)) 
+	p.class = p / nclass
+	sample.sizes <- vector()
+	  for(s in 1:nclass) { 
+	    sample.sizes[s] <- round(length(x$y[x$y == unique(x$y)[s]]) * 
+		                         p.class, digits=0)    
+	  } 
+	
+      for(i in 1:n) {
+	    if(trace) cat("running iteration:", i, "\n")
+        # Draw random sample		
+	    if(!bootstrap) {
+          sidx <- list()				
+            for(s in 1:nclass) {
+              sidx[[s]] <- sample(which(x$y %in% unique(x$y)[s]), sample.sizes[s])
+            }
+			sidx <- unlist(sidx)  
+              tx <- xdata[sidx,]
+              ty <- x$y[sidx]
+              mx <- xdata[-sidx,]
+              my <- x$y[-sidx]	           
+	    } else {	
+          dat <- data.frame(y=x$y, xdata) 	
+            tx <- dat[sample(1:nrow(dat), replace=TRUE),]
+              ty <- tx$y 
+                tx <- tx[,2:ncol(tx)]
+              mx <- dat[-which(!rownames(dat) %in% rownames(tx)),]
+            my <- mx$y
+          mx <- mx[,2:ncol(mx)] 
+       }
+      rf.fit <- randomForest::randomForest(y=as.factor(my), x = mx, ytest=as.factor(ty), xtest=tx, ...)        
+      options(warn=-1)
       cv.acc <- accuracy(rf.fit$test$predicted,  ty)
 	    if(bootstrap) boot.sample.size <- append(boot.sample.size, length(my))
-        if(!length(classes) == length(unique(ty))) {
-	      #### add code to check for presence of classes and set to NA if absent classes occur   
-	    }		  
-		cv.ua <- rbind(cv.ua, cv.acc$users.accuracy) 
+ 		cv.ua <- rbind(cv.ua, cv.acc$users.accuracy) 
 		  cv.pa <- rbind(cv.pa, cv.acc$producers.accuracy) 
 	        cv.oob <- rbind(mdl.oob, c(apply(rf.fit$test$err.rate, MARGIN = 2, stats::median),
 		                    cv.acc$kappa))	  
@@ -207,7 +225,7 @@ rf.crossValidation <- function(x, xdata, p=0.10, n=99, seed=NULL, normalize = FA
 		  mdl.pa <- rbind(mdl.pa, mdl.acc$producers.accuracy) 
             mdl.oob <- rbind(mdl.oob, c(apply(rf.fit$err.rate, MARGIN = 2, stats::median),
 		                     mdl.acc$kappa))
-
+	options(warn=0)
 	}  
     names(cv.ua) <- c(classes) 
       names(cv.pa)  <- c(classes) 
